@@ -28,9 +28,11 @@
 
 #include <string.h>
 
+#include <libpeas/peas-engine.h>
 #include <libpeas/peas-i18n.h>
 
 #include "peas-gtk-plugin-manager-view.h"
+#include "peas-gtk-disable-plugins-dialog.h"
 #include "peas-gtk-plugin-manager-store.h"
 #include "peas-gtk-configurable.h"
 
@@ -63,6 +65,7 @@ struct _PeasGtkPluginManagerViewPrivate {
 /* Properties */
 enum {
   PROP_0,
+  PROP_ENGINE,
   PROP_SHOW_BUILTIN
 };
 
@@ -117,6 +120,73 @@ convert_child_iter_to_iter (PeasGtkPluginManagerView *view,
   return success;
 }
 
+static GList *
+get_dependant_plugins (PeasGtkPluginManagerView *view,
+                       PeasPluginInfo           *info)
+{
+  const gchar *module_name;
+  const GList *plugins;
+  GList *dep_plugins = NULL;
+
+  module_name = peas_plugin_info_get_module_name (info);
+  plugins = peas_engine_get_plugin_list (view->priv->engine);
+
+  for (; plugins != NULL; plugins = plugins->next)
+    {
+      PeasPluginInfo *plugin = (PeasPluginInfo *) plugins->data;
+
+      if (!peas_plugin_info_is_loaded (plugin))
+        continue;
+
+      /* Don't add builtin plugins if they are not shown */
+      if (!view->priv->show_builtin && peas_plugin_info_is_builtin (plugin))
+        continue;
+
+      if (peas_plugin_info_has_dependency (plugin, module_name))
+        dep_plugins = g_list_prepend (dep_plugins, plugin);
+    }
+
+  return dep_plugins;
+}
+
+static void
+toggle_enabled (PeasGtkPluginManagerView *view,
+                GtkTreeIter              *iter)
+{
+  PeasPluginInfo *info;
+
+  info = peas_gtk_plugin_manager_store_get_plugin (view->priv->store, iter);
+
+  if (peas_plugin_info_is_loaded (info))
+    {
+      GList *dep_plugins;
+
+      dep_plugins = get_dependant_plugins (view, info);
+
+      if (dep_plugins != NULL)
+        {
+          GtkWindow *parent;
+          GtkWidget *dialog;
+          gint response;
+
+          parent = GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (view)));
+
+          /* The dialog takes the list so don't free it */
+          dialog = peas_gtk_disable_plugins_dialog_new (parent, info,
+                                                        dep_plugins);
+
+          response = gtk_dialog_run (GTK_DIALOG (dialog));
+
+          gtk_widget_destroy (dialog);
+
+          if (response != GTK_RESPONSE_OK)
+            return;
+        }
+    }
+
+  peas_gtk_plugin_manager_store_toggle_enabled (view->priv->store, iter);
+}
+
 static void
 plugin_list_changed_cb (PeasEngine               *engine,
                         GParamSpec               *pspec,
@@ -183,7 +253,7 @@ enabled_toggled_cb (GtkCellRendererToggle    *cell,
   if (gtk_tree_model_get_iter (model, &iter, path))
     {
       convert_iter_to_child_iter (view, &iter);
-      peas_gtk_plugin_manager_store_toggle_enabled (view->priv->store, &iter);
+      toggle_enabled (view, &iter);
     }
 
   gtk_tree_path_free (path);
@@ -243,7 +313,7 @@ enabled_menu_cb (GtkMenu                  *menu,
 
   convert_iter_to_child_iter (view, &iter);
 
-  peas_gtk_plugin_manager_store_toggle_enabled (view->priv->store, &iter);
+  toggle_enabled (view, &iter);
 }
 
 static void
@@ -278,7 +348,7 @@ create_popup_menu (PeasGtkPluginManagerView *view)
   gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (item),
                                   peas_plugin_info_is_loaded (info));
   g_signal_connect (item, "toggled", G_CALLBACK (enabled_menu_cb), view);
-  gtk_widget_set_sensitive (item, peas_plugin_info_is_available (info) &&
+  gtk_widget_set_sensitive (item, peas_plugin_info_is_available (info, NULL) &&
                                   !peas_plugin_info_is_builtin (info));
   gtk_menu_shell_append (GTK_MENU_SHELL (menu), item);
 
@@ -447,13 +517,6 @@ peas_gtk_plugin_manager_view_init (PeasGtkPluginManagerView *view)
                                             PEAS_GTK_TYPE_PLUGIN_MANAGER_VIEW,
                                             PeasGtkPluginManagerViewPrivate);
 
-  view->priv->store = peas_gtk_plugin_manager_store_new ();
-  view->priv->engine = g_object_ref (peas_engine_get_default ());
-  g_signal_connect (view->priv->engine,
-                    "notify::plugin-list",
-                    G_CALLBACK (plugin_list_changed_cb),
-                    view);
-
   gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (view), TRUE);
   gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (view), FALSE);
 
@@ -508,10 +571,6 @@ peas_gtk_plugin_manager_view_init (PeasGtkPluginManagerView *view)
                                        (GtkTreeViewSearchEqualFunc) name_search_cb,
                                        view,
                                        NULL);
-
-  /* Properly set the model */
-  view->priv->show_builtin = TRUE;
-  peas_gtk_plugin_manager_view_set_show_builtin (view, FALSE);
 }
 
 static gboolean
@@ -545,6 +604,51 @@ peas_gtk_plugin_manager_view_popup_menu (GtkWidget *widget)
   return TRUE;
 }
 
+static gboolean
+peas_gtk_plugin_manager_view_query_tooltip (GtkWidget  *widget,
+                                            gint        x,
+                                            gint        y,
+                                            gboolean    keyboard_mode,
+                                            GtkTooltip *tooltip)
+{
+  PeasGtkPluginManagerView *view = PEAS_GTK_PLUGIN_MANAGER_VIEW (widget);
+  gboolean is_row;
+  GtkTreeIter iter;
+  PeasPluginInfo *info;
+  GError *error = NULL;
+
+  is_row = gtk_tree_view_get_tooltip_context (GTK_TREE_VIEW (widget),
+                                              &x, &y, keyboard_mode,
+                                              NULL, NULL, &iter);
+
+  if (!is_row)
+    return FALSE;
+    
+  convert_iter_to_child_iter (view, &iter);
+
+  info = peas_gtk_plugin_manager_store_get_plugin (view->priv->store, &iter);
+
+  if (!peas_plugin_info_is_available (info, &error))
+    {
+      gchar *message;
+
+      message = g_strdup_printf (_("<b>The plugin '%s' could not be loaded</b>\n"
+                                   "An error occurred: %s"),
+                                 peas_plugin_info_get_name (info),
+                                 error->message);
+
+      gtk_tooltip_set_markup (tooltip, message);
+
+      g_free (message);
+      g_error_free (error);
+
+      return TRUE;
+    }
+
+  return FALSE;
+}
+  
+
 static void
 peas_gtk_plugin_manager_view_row_activated (GtkTreeView       *tree_view,
                                             GtkTreePath       *path,
@@ -559,7 +663,7 @@ peas_gtk_plugin_manager_view_row_activated (GtkTreeView       *tree_view,
   convert_iter_to_child_iter (view, &iter);
 
   if (peas_gtk_plugin_manager_store_can_enable (view->priv->store, &iter))
-    peas_gtk_plugin_manager_store_toggle_enabled (view->priv->store, &iter);
+    toggle_enabled (view, &iter);
 }
 
 static void
@@ -572,6 +676,9 @@ peas_gtk_plugin_manager_view_set_property (GObject      *object,
 
   switch (prop_id)
     {
+    case PROP_ENGINE:
+      view->priv->engine = g_value_get_object (value);
+      break;
     case PROP_SHOW_BUILTIN:
       peas_gtk_plugin_manager_view_set_show_builtin (view,
                                                      g_value_get_boolean (value));
@@ -592,6 +699,9 @@ peas_gtk_plugin_manager_view_get_property (GObject    *object,
 
   switch (prop_id)
     {
+    case PROP_ENGINE:
+      g_value_set_object (value, view->priv->engine);
+      break;
     case PROP_SHOW_BUILTIN:
       g_value_set_boolean (value,
                            peas_gtk_plugin_manager_view_get_show_builtin (view));
@@ -602,6 +712,31 @@ peas_gtk_plugin_manager_view_get_property (GObject    *object,
     }
 }
 
+static void
+peas_gtk_plugin_manager_view_constructed (GObject *object)
+{
+  PeasGtkPluginManagerView *view = PEAS_GTK_PLUGIN_MANAGER_VIEW (object);
+
+  if (view->priv->engine == NULL)
+    view->priv->engine = peas_engine_get_default ();
+
+  g_object_ref (view->priv->engine);
+
+  view->priv->store = peas_gtk_plugin_manager_store_new (view->priv->engine);
+
+  /* Properly set the model */
+  view->priv->show_builtin = TRUE;
+  peas_gtk_plugin_manager_view_set_show_builtin (view, FALSE);
+
+  g_signal_connect (view->priv->engine,
+                    "notify::plugin-list",
+                    G_CALLBACK (plugin_list_changed_cb),
+                    view);
+
+  if (G_OBJECT_CLASS (peas_gtk_plugin_manager_view_parent_class)->constructed != NULL)
+    G_OBJECT_CLASS (peas_gtk_plugin_manager_view_parent_class)->constructed (object);
+}
+
 static void
 peas_gtk_plugin_manager_view_dispose (GObject *object)
 {
@@ -642,13 +777,30 @@ peas_gtk_plugin_manager_view_class_init (PeasGtkPluginManagerViewClass *klass)
 
   object_class->set_property = peas_gtk_plugin_manager_view_set_property;
   object_class->get_property = peas_gtk_plugin_manager_view_get_property;
+  object_class->constructed = peas_gtk_plugin_manager_view_constructed;
   object_class->dispose = peas_gtk_plugin_manager_view_dispose;
 
   widget_class->button_press_event = peas_gtk_plugin_manager_view_button_press_event;
   widget_class->popup_menu = peas_gtk_plugin_manager_view_popup_menu;
+  widget_class->query_tooltip = peas_gtk_plugin_manager_view_query_tooltip;
 
   tree_view_class->row_activated = peas_gtk_plugin_manager_view_row_activated;
 
+  /**
+   * PeasGtkPLuginManagerView:engine:
+   *
+   * The #PeasEngine this view is attached to.
+   */
+  g_object_class_install_property (object_class,
+                                   PROP_ENGINE,
+                                   g_param_spec_object ("engine",
+                                                        "engine",
+                                                        "The PeasEngine this view is attached to",
+                                                        PEAS_TYPE_ENGINE,
+                                                        G_PARAM_READWRITE |
+                                                        G_PARAM_CONSTRUCT_ONLY |
+                                                        G_PARAM_STATIC_STRINGS));
+
   /**
    * PeasGtkPluginManagerView:show-builtin:
    *
@@ -688,15 +840,21 @@ peas_gtk_plugin_manager_view_class_init (PeasGtkPluginManagerViewClass *klass)
 
 /**
  * peas_gtk_plugin_manager_view_new:
+ * @engine: (allow-none): A #PeasEngine, or %NULL.
  *
- * Creates a new plugin manager view.
+ * Creates a new plugin manager view for the given #PeasEngine.
+ *
+ * If @engine is %NULL, then the default engine will be used.
  *
  * Returns: the new #PeasGtkPluginManagerView.
  */
 GtkWidget *
-peas_gtk_plugin_manager_view_new (void)
+peas_gtk_plugin_manager_view_new (PeasEngine *engine)
 {
+  g_return_val_if_fail (engine == NULL || PEAS_IS_ENGINE (engine), NULL);
+
   return GTK_WIDGET (g_object_new (PEAS_GTK_TYPE_PLUGIN_MANAGER_VIEW,
+                                   "engine", engine,
                                    NULL));
 }
 
