@@ -23,6 +23,7 @@
 #define __PEAS_EXTENSION_H__
 
 #include <glib-object.h>
+#include <girepository.h>
 
 G_BEGIN_DECLS
 
@@ -36,26 +37,41 @@ G_BEGIN_DECLS
 #define PEAS_IS_EXTENSION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), PEAS_TYPE_EXTENSION))
 #define PEAS_EXTENSION_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS((obj), PEAS_TYPE_EXTENSION, PeasExtensionClass))
 
+typedef struct _PeasExtension         PeasExtension;
+typedef struct _PeasExtensionClass    PeasExtensionClass;
+typedef struct _PeasExtensionPrivate  PeasExtensionPrivate;
+
 /**
  * PeasExtension:
- * @parent: the parent object.
  *
- * Base class for plugins.
+ * The #PeasExtension structure contains only private data and should only be
+ * accessed using the provided API.
  */
-typedef struct _PeasExtension       PeasExtension;
-typedef struct _PeasExtensionClass  PeasExtensionClass;
-
 struct _PeasExtension {
   GObject parent;
+
+  /*< private >*/
+  PeasExtensionPrivate *priv;
 };
 
+/**
+ * PeasExtensionClass:
+ *
+ * The #PeasExtensionClass structure contains only private data and should
+ * only be accessed using the provided API.  You should not inherit from this
+ * class.
+ */
 struct _PeasExtensionClass {
   GObjectClass parent_class;
 
-  /* Virtual public methods */
+  /*< private >*/
   gboolean   (*call)                      (PeasExtension  *exten,
+                                           GType           gtype,
                                            const gchar    *method,
-                                           va_list         args);
+                                           GIArgument     *args,
+                                           GIArgument     *return_value);
+
+  gpointer padding[8];
 };
 
 /*
@@ -63,12 +79,19 @@ struct _PeasExtensionClass {
  */
 GType        peas_extension_get_type        (void)  G_GNUC_CONST;
 
+GType        peas_extension_get_extension_type
+                                            (PeasExtension *exten);
+
 gboolean     peas_extension_call            (PeasExtension *exten,
                                              const gchar   *method_name,
                                              ...);
 gboolean     peas_extension_call_valist     (PeasExtension *exten,
                                              const gchar   *method_name,
                                              va_list        args);
+gboolean     peas_extension_callv           (PeasExtension *exten,
+                                             const gchar   *method_name,
+                                             GIArgument    *args,
+                                             GIArgument    *return_value);
 
 G_END_DECLS
 
